/* ------------------------------------------------ *
 * The MIT License (MIT)
 * Copyright (c) 2019 terryky1220@gmail.com
 * ------------------------------------------------ */
#include <stdio.h>
#include <GLES2/gl2.h>
#include "util_shader.h"
#include "util_debugstr.h"
#include "assertgl.h"

#define UNUSED(x) (void)(x)

static const char s_strDbgStrVS[] =
    "attribute vec4 a_Vertex;                               \n"
    "attribute vec2 a_UV;                                   \n"
    "uniform   vec4 u_Translate;                            \n"
    "uniform   vec4 u_PrjMul, u_PrjAdd;                     \n"
    "varying   vec2 v_UV;                                   \n"
    "                                                       \n"
    "void main()                                            \n"
    "{                                                      \n"
    "   vec4 pos;                                           \n"
    "   pos         = a_Vertex + u_Translate;               \n"
    "   pos         = pos * u_PrjMul;                       \n"
    "   gl_Position = pos + u_PrjAdd;                       \n"
    "   v_UV        = a_UV;                                 \n"
    "}                                                      \n";

static const char s_strDbgStrFS[] =
    "precision mediump float;                               \n"
    "                                                       \n"
    "varying vec2       v_UV;                               \n"
    "uniform sampler2D  u_Sampler0;                         \n"
    "uniform vec4       u_ColorFG;                          \n"
    "uniform vec4       u_ColorBG;                          \n"
    "                                                       \n"
    "void main()                                            \n"
    "{                                                      \n"
    "   vec4 texcol = texture2D (u_Sampler0, v_UV);         \n"
    "   gl_FragColor = mix (u_ColorBG, u_ColorFG, texcol.r);\n"
    "}                                                      \n";


#define DEBSTR_FONT_WIDTH   (12)
#define DEBSTR_FONT_HEIGHT  (22)
#define DEBSTR_FONT_NUM     (95)
#define DBGSTR_IMAGE_WIDTH  (DEBSTR_FONT_WIDTH*DEBSTR_FONT_NUM)
#define DBGSTR_IMAGE_HEIGHT (DEBSTR_FONT_HEIGHT)

static unsigned int     s_font12x22[];
static unsigned int     s_fontTexID;
static int              s_wndW, s_wndH;
static GLuint           s_progShader;
static int              locVtx, locUv, locTranslate, locSampler0;
static int              locPrjMul, locPrjAdd;
static int              locColFG, locColBG;

static int
load_debug_font_texture (void)
{
    glGenTextures (1, &s_fontTexID);
    glBindTexture (GL_TEXTURE_2D, s_fontTexID);
    glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR      );
    glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR      );
    glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     GL_CLAMP_TO_EDGE);
    glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     GL_CLAMP_TO_EDGE);

    glPixelStorei (GL_UNPACK_ALIGNMENT, 4);

    glTexImage2D (GL_TEXTURE_2D, 0, GL_LUMINANCE, 
                      DBGSTR_IMAGE_WIDTH, DBGSTR_IMAGE_HEIGHT,
                      0, GL_LUMINANCE, 
                      GL_UNSIGNED_BYTE,
                      s_font12x22);
    return 0;
}


static int
setup_shader() 
{
    s_progShader = build_shader (s_strDbgStrVS, s_strDbgStrFS);

    locVtx = glGetAttribLocation (s_progShader, "a_Vertex");
    locUv  = glGetAttribLocation (s_progShader, "a_UV"    );

    locSampler0  = glGetUniformLocation (s_progShader, "u_Sampler0" );
    locTranslate = glGetUniformLocation (s_progShader, "u_Translate");
    locPrjMul    = glGetUniformLocation (s_progShader, "u_PrjMul");
    locPrjAdd    = glGetUniformLocation (s_progShader, "u_PrjAdd");
    locColFG     = glGetUniformLocation (s_progShader, "u_ColorFG" );
    locColBG     = glGetUniformLocation (s_progShader, "u_ColorBG" );

    return 0;
}




void
init_dbgstr (int win_w, int win_h)
{
    s_wndW = win_w;
    s_wndH = win_h;

    load_debug_font_texture ();
    setup_shader();
}


int
draw_dbgstr_ex (char *str, int x, int y, float scale, float *col_fg, float *col_bg)
{
    int   i, row, column;
    float u0, u1;
    float vertices[8], texcoord[8];
    float fW = DEBSTR_FONT_WIDTH  * scale;
    float fH = DEBSTR_FONT_HEIGHT * scale;
    float fx = (float)x;
    float fy = (float)y;

    vertices[ 2 * 0 + 0 ] = fx;      vertices[ 2 * 0 + 1 ] = fy;
    vertices[ 2 * 1 + 0 ] = fx;      vertices[ 2 * 1 + 1 ] = fy + fH;
    vertices[ 2 * 2 + 0 ] = fx + fW; vertices[ 2 * 2 + 1 ] = fy;
    vertices[ 2 * 3 + 0 ] = fx + fW; vertices[ 2 * 3 + 1 ] = fy + fH;

    glUseProgram (s_progShader);

    glBindBuffer (GL_ARRAY_BUFFER, 0);
    glBindBuffer (GL_ELEMENT_ARRAY_BUFFER, 0);

    glEnableVertexAttribArray (locVtx);
    glEnableVertexAttribArray (locUv );
    glVertexAttribPointer (locVtx, 2, GL_FLOAT, GL_FALSE, 0, vertices);
    glVertexAttribPointer (locUv,  2, GL_FLOAT, GL_FALSE, 0, texcoord);

    glUniform1i (locSampler0, 0);
    glUniform4f (locPrjMul, 2.0f / s_wndW, -2.0f / s_wndH, 0.0f, 0.0f);
    glUniform4f (locPrjAdd, -1.0f, 1.0f, 1.0f, 1.0f);
    glUniform4fv(locColFG, 1, col_fg);
    glUniform4fv(locColBG, 1, col_bg);

    glDisable (GL_DEPTH_TEST);
    glDisable (GL_CULL_FACE );
    glActiveTexture (GL_TEXTURE0);
    glBindTexture   (GL_TEXTURE_2D, s_fontTexID);
    GLASSERT();

    glEnable (GL_BLEND);
    glBlendFuncSeparate (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, 
    	       GL_ONE, GL_ONE_MINUS_SRC_ALPHA);

    row = column = 0;
    for (i = 0; ; i ++)
    {
        int c = str[i];

        if ( c == 0 )
            break;

        if (c == '\n')
        {
            row ++;
            column = 0;
            continue;
        }

        u0 = (c - 0x20) * (1.0f / DEBSTR_FONT_NUM);
        u1 = u0 + (1.0f / DEBSTR_FONT_NUM);

        texcoord[ 2 * 0 + 0 ] = texcoord[ 2 * 1 + 0 ] = u0;
        texcoord[ 2 * 2 + 0 ] = texcoord[ 2 * 3 + 0 ] = u1;
        texcoord[ 2 * 0 + 1 ] = texcoord[ 2 * 2 + 1 ] = 0.0;
        texcoord[ 2 * 1 + 1 ] = texcoord[ 2 * 3 + 1 ] = 1.0;

        glUniform4f (locTranslate, column * fW, row * fH, 0.0f, 0.0f );

        glDrawArrays (GL_TRIANGLE_STRIP, 0, 4);

        column ++;
    }

    glDisable (GL_BLEND);
    glDisableVertexAttribArray (locVtx);
    glDisableVertexAttribArray (locUv );
    GLASSERT();

    return 0;
}


int
draw_dbgstr (char *str, int x, int y)
{
    float col_fg[] = {1.0f, 1.0f, 1.0f, 1.0f};
    float col_bg[] = {0.0f, 0.0f, 0.0f, 0.5f};
    float scale = 1.0f;

    return draw_dbgstr_ex (str, x, y, scale, col_fg, col_bg);
}


/*
 *  Pixel array data of Font Texture.
 *      - this texture is generated with "Bitstream-Vera-Sans-Mono" font.
 *      - texture width  is (12 * 95) [pix]
 *      - texture height is (22 *  1) [pix]
 *      - each pixel contains 8bit alpha values.
 *
 *  |<---------------------------  12 [pix] x 95 [chars] = 1140 [pix]  ---------------------------->|
 *
 *  +-----------------------------------------------------------------------------------------------+
 *  |0        1         2         3         4         5         6         7         8         9     |
 *  |12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345|
 *  +-----------------------------------------------------------------------------------------------+
 *  | !"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~| 22 [pix]
 *  +-----------------------------------------------------------------------------------------------+
 *
 */
static unsigned int
s_font12x22[] ={
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xe9aa0200,0x0000000e,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x04000000,0x000ee9cf,0x00000000,
    0xffcf0400,0xff5e0076,0x00000ee9,0x04000000,0x040076cf,0x000076cf,0x00000000,0x0007bd1f,
    0x00000000,0xc9ffe743,0x0000002f,0x00000000,0xba1f0000,0x76ffffff,0x00000000,0x00000000,
    0x0ee9ff5e,0x00000000,0x00000000,0x51e71f00,0x00000000,0xe71f0000,0x00000051,0x00000000,
    0x00000000,0x0000765e,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x5e000000,0x000076ff,0xba1d0000,0x2fc9ffff,0x00000000,0x90420000,0x009cffff,0x00000000,
    0xffb94200,0x29c2ffff,0x00000000,0xe7902600,0x4ec2ffff,0x00000000,0x00000000,0xf7ff5e00,
    0x0000002f,0xffffcf04,0xffffffff,0x0000009c,0x68020000,0x9dffffe7,0x00000029,0xffffff84,
    0xffffffff,0x00009cff,0xc8430000,0x75cdffff,0x00000007,0xe7680200,0x4ec9ffff,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xc8680200,
    0x54f7ffff,0x00000000,0x00000000,0x00000000,0x00000000,0x1f000000,0x0076ffe7,0x02000000,
    0xffffffaa,0x75cdffff,0x00000007,0x68020000,0xcdffffc8,0x02000754,0xffffffaa,0x0754c6ff,
    0x00000000,0xffffcf04,0xffffffff,0x00009cff,0xffff8400,0xffffffff,0x0007c2ff,0x43020000,
    0xf7ffffc8,0x02002299,0x002ff7aa,0x5e000000,0x000076ff,0xffffcf04,0xffffffff,0x00000ee9,
    0xff390000,0xffffffff,0x02000051,0x002ff7aa,0x1f000000,0x002ff7e7,0x2ff7aa02,0x00000000,
    0x02000000,0x0ee9ffaa,0xaa020000,0x022ff7ff,0x07c2ffaa,0x5e000000,0x000076ff,0xba420000,
    0x54cdffff,0x00000000,0xffffcf04,0xc2ffffff,0x00000029,0xba420000,0x54cdffff,0x02000000,
    0xffffffaa,0x2cc2ffff,0x00000000,0xc8680200,0x75cdffff,0x5e000007,0xffffffff,0xffffffff,
    0x029cffff,0x002ff7aa,0x84000000,0x000051ff,0x0051ff84,0x00000000,0xffffe71f,0x00002ff7,
    0x00000000,0x10e9cf04,0x0076ffaa,0x02000000,0x3976ffaa,0x0007c2ff,0x02000000,0x0051ffaa,
    0xffffff84,0xffffffff,0x0007c2ff,0x5e000000,0xc2ffffff,0x02000007,0x002ff7aa,0x00000000,
    0x00000000,0xff840000,0x07c2ffff,0x00000000,0x39000000,0x0051ffff,0x00000000,0x00000000,
    0x00000000,0x00000000,0xcf040000,0x0000009c,0x00000000,0x00000000,0x00000000,0x00000000,
    0x0051ff84,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x39000000,
    0x0007c2ff,0x00000000,0x00000000,0x00000000,0x00000000,0xffff9002,0x000076ff,0x00000000,
    0x00000000,0x00000000,0x0051ff84,0x00000000,0x00000000,0x00000000,0x0051ff84,0x00000000,
    0x00000000,0x07c2e71f,0x00000000,0x51ff8400,0x00000000,0x00000000,0xffffff5e,0x000076ff,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0xffc85d00,0x00000ee9,0x02000000,0x0007c2aa,
    0x00000000,0xf7ffaa02,0x0000079d,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x04000000,0x000ee9cf,0x00000000,0xffcf0400,0xff5e0076,0x00000ee9,
    0x39000000,0x390051ff,0x000051ff,0x00000000,0x0007bd1f,0x39000000,0x840276ff,0x00002ff7,
    0x00000000,0xffe71f00,0x76ffffff,0x00000000,0x00000000,0x0ee9ff5e,0x00000000,0x00000000,
    0x07c28400,0x00000000,0x84000000,0x000007c2,0x00000000,0x00000000,0x0000765e,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xcf040000,0x00000ee9,0xffe71f00,
    0xffffffff,0x00000051,0xffff8400,0x009cffff,0x00000000,0xffffff39,0xffffffff,0x00000051,
    0xffffff39,0xffffffff,0x00000076,0x00000000,0xf7ffcf04,0x0000002f,0xffffcf04,0xffffffff,
    0x0000009c,0xffaa0200,0xffffffff,0x0000009c,0xffffff84,0xffffffff,0x000076ff,0xffff8400,
    0xffffffff,0x0000009c,0xffffaa02,0xffffffff,0x00000051,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0xffff8400,0xffffffff,0x00000076,0x43020000,
    0xf7ffffc8,0x00000775,0x5e000000,0x07c2ffff,0x02000000,0xffffffaa,0xffffffff,0x000007c2,
    0xffaa0200,0xffffffff,0x020051ff,0xffffffaa,0xf7ffffff,0x0000002f,0xffffcf04,0xffffffff,
    0x00009cff,0xffff8400,0xffffffff,0x0007c2ff,0xffaa0200,0xffffffff,0x02009cff,0x002ff7aa,
    0x5e000000,0x000076ff,0xffffcf04,0xffffffff,0x00000ee9,0xff390000,0xffffffff,0x02000051,
    0x002ff7aa,0xe71f0000,0x00002ff7,0x2ff7aa02,0x00000000,0x02000000,0x51ffffaa,0xcf040000,
    0x022ff7ff,0x51ffffaa,0x5e000000,0x000076ff,0xffff3900,0xffffffff,0x00000099,0xffffcf04,
    0xffffffff,0x000051f7,0xffff3900,0xffffffff,0x0200009c,0xffffffaa,0xffffffff,0x00000054,
    0xffffaa02,0xffffffff,0x5e00009c,0xffffffff,0xffffffff,0x029cffff,0x002ff7aa,0x84000000,
    0x000051ff,0x009cff5e,0x00000000,0xaaa1ff5e,0x00002ff7,0x00000000,0x07c2e71f,0x2ff7ff39,
    0x5e000000,0x0207c2ff,0x0076ffaa,0x5e000000,0x0007c2ff,0xffffff84,0xffffffff,0x0007c2ff,
    0x5e000000,0xc2ffffff,0x00000007,0x009cff39,0x00000000,0x00000000,0xff840000,0x07c2ffff,
    0x00000000,0xe71f0000,0x2ff7c8cd,0x00000000,0x00000000,0x00000000,0x00000000,0x39000000,
    0x000051ff,0x00000000,0x00000000,0x00000000,0x00000000,0x0051ff84,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x39000000,0x0007c2ff,0x00000000,0x00000000,
    0x00000000,0x02000000,0xffffffaa,0x000076ff,0x00000000,0x00000000,0x00000000,0x0051ff84,
    0x00000000,0x00000000,0x00000000,0x0051ff84,0x00000000,0x00000000,0x07c2e71f,0x00000000,
    0x51ff8400,0x00000000,0x00000000,0xffffff5e,0x000076ff,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0xaa020000,0x00002ff7,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0xffffff5e,0x00000ee9,0x02000000,0x0007c2aa,0x00000000,0xffffaa02,0x00009cff,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x04000000,
    0x000ee9cf,0x00000000,0xffe71f00,0xff840051,0x000007c2,0x5e000000,0x5e000ee9,0x00000ee9,
    0xba420000,0x9df7ffff,0x84000029,0x39002ff7,0x000076ff,0x00000000,0xa1ff8400,0x00000007,
    0x00000000,0x00000000,0x07c2ff84,0x00000000,0x00000000,0x0076e71f,0x00000000,0x39000000,
    0x000076ff,0x00000000,0x0029c21f,0x1d00765e,0x00002fb9,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0xff5e0000,0x00000076,0x9cffaa02,0xff690207,0x000007c2,0x4e9d8400,
    0x009cff39,0x00000000,0x002c9d39,0xff830200,0x00000ee9,0x072cc239,0xff430000,0x00002ff7,
    0x00000000,0xf7bac984,0x0000002f,0x0ee9cf04,0x00000000,0x00000000,0xf7ff8400,0x26000054,
    0x00000075,0x00000000,0xcf040000,0x00002ff7,0x54ffe71f,0xe7420000,0x00002ff7,0x4ef7ff39,
    0xe7420000,0x00000ee9,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x22758300,0xff430000,0x00000ee9,0xc9ba0400,0x4300004e,0x00009cff,0xaa020000,
    0x2ff7ffff,0x02000000,0x002ff7aa,0xcf1d0000,0x000076ff,0xc9ff8400,0x1b000029,0x02005183,
    0x002ff7aa,0xffba2600,0x00000ee9,0x0ee9cf04,0x00000000,0x00000000,0x51ff8400,0x00000000,
    0x00000000,0xc9ff8400,0x02000032,0x02007a43,0x002ff7aa,0x5e000000,0x000076ff,0x04000000,
    0x000ee9cf,0x00000000,0x00000000,0xff840000,0x02000051,0x002ff7aa,0xf7cf0400,0x00000051,
    0x2ff7aa02,0x00000000,0x02000000,0x76c8f7aa,0xf7390000,0x022ff7c8,0x9cffffaa,0x5e000000,
    0x000076ff,0x7affcf04,0xe7420007,0x000051ff,0x0ee9cf04,0xaa1d0000,0x0007c2ff,0x7affcf04,
    0xe7420007,0x020051ff,0x002ff7aa,0xff830200,0x00000ee9,0x4ef7ff5e,0x681b0000,0x0000007a,
    0x04000000,0x000ee9cf,0x02000000,0x002ff7aa,0x84000000,0x000051ff,0x07c2e71f,0x00000000,
    0x8451ff84,0x000051ff,0x00000000,0x009cff39,0x9cff8400,0xe71f0000,0x000051ff,0x0ee9ff39,
    0xcf040000,0x000051ff,0x00000000,0xcf040000,0x000051ff,0x5e000000,0x000076ff,0x00000000,
    0x2ff7aa02,0x00000000,0x00000000,0x00000000,0x07c2ff39,0x00000000,0xc9e71f00,0xf7ba040e,
    0x0000002f,0x00000000,0x00000000,0x00000000,0x00000000,0x002ff75e,0x00000000,0x00000000,
    0x00000000,0x00000000,0x0051ff84,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x39000000,0x0007c2ff,0x00000000,0x00000000,0x00000000,0x1f000000,0x0029e9e7,
    0x00000000,0x00000000,0x00000000,0x00000000,0x0051ff84,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x51ff8400,0x00000000,0x00000000,
    0x5e000000,0x000076ff,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0xaa020000,0x00002ff7,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x0076ff84,0x00000000,
    0x02000000,0x0007c2aa,0x00000000,0x43000000,0x000ee9e7,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x04000000,0x000ee9cf,0x00000000,0xc2ff3900,
    0xffaa0207,0x00000051,0xaa020000,0xaa0207c2,0x0000009c,0xffff5e00,0xffffffff,0x840007c2,
    0x39002ff7,0x000076ff,0x00000000,0x51ff8400,0x00000000,0x00000000,0x02000000,0x0051ffaa,
    0x00000000,0x00000000,0x002ff784,0x00000000,0x04000000,0x0007c2cf,0x00000000,0x75e36802,
    0xe3687a69,0x00000775,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xe9cf0400,
    0x0000000e,0x0ee9e71f,0xaa020000,0x000051ff,0x00000000,0x009cff39,0x00000000,0x00000000,
    0xaa020000,0x00002ff7,0x00000000,0x84000000,0x000051ff,0x1f000000,0xf7aa32e3,0x0000002f,
    0x0ee9cf04,0x00000000,0x00000000,0x51ffe71f,0x00000000,0x00000000,0x00000000,0xff390000,
    0x0000009c,0x009cff39,0x84000000,0x000051ff,0x0076ff84,0x39000000,0x000051ff,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xaa020000,
    0x00002ff7,0x0ee9aa02,0x00000000,0x0051ff5e,0xe71f0000,0x51ff9ce9,0x02000000,0x002ff7aa,
    0x39000000,0x00009cff,0x0ee9e71f,0x00000000,0x02000000,0x002ff7aa,0x84000000,0x000076ff,
    0x0ee9cf04,0x00000000,0x00000000,0x51ff8400,0x00000000,0x00000000,0x0ee9e71f,0x00000000,
    0x02000000,0x002ff7aa,0x5e000000,0x000076ff,0x04000000,0x000ee9cf,0x00000000,0x00000000,
    0xff840000,0x02000051,0x002ff7aa,0x51ffcf04,0x00000000,0x2ff7aa02,0x00000000,0x02000000,
    0xc29bf7aa,0xa1840007,0x022ff7aa,0xe99bf7aa,0x5e00000e,0x000076ff,0x009cff39,0x5e000000,
    0x00009cff,0x0ee9cf04,0x04000000,0x002ff7cf,0x009cff39,0x5e000000,0x02009cff,0x002ff7aa,
    0xaa020000,0x02002ff7,0x0051ffaa,0x00000000,0x00000000,0x04000000,0x000ee9cf,0x02000000,
    0x002ff7aa,0x84000000,0x000051ff,0x2ff7aa02,0x04000000,0x5e0ee9cf,0x000076ff,0x00000000,
    0x0076ff5e,0xffcf0400,0xff840051,0x0000009c,0x9cff8400,0xff840000,0x0000009c,0x00000000,
    0xff840000,0x0000009c,0x5e000000,0x000076ff,0x00000000,0x9cff3900,0x00000000,0x00000000,
    0x00000000,0x07c2ff39,0x00000000,0x0ec9cf04,0xba040000,0x00000ee9,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x0051ff84,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x39000000,0x0007c2ff,
    0x00000000,0x00000000,0x00000000,0x1f000000,0x0007c2e7,0x00000000,0x00000000,0x00000000,
    0x00000000,0x0051ff84,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x51ff8400,0x00000000,0x00000000,0x5e000000,0x000076ff,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xaa020000,0x00002ff7,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x02000000,0x002ff7aa,0x00000000,0x02000000,0x0007c2aa,0x00000000,
    0x04000000,0x000ee9cf,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x04000000,0x000ee9cf,0x00000000,0x51ff8400,0xc2e71f00,0x02000007,0xffffffaa,
    0xffffffff,0x0ee9ffff,0x71ffcf04,0x4209bd1f,0x39000796,0x840276ff,0x00002ff7,0x00074000,
    0x76ff5e00,0x00000000,0x00000000,0x1f000000,0x0007c2e7,0x00000000,0x04000000,0x0007c2cf,
    0x00000000,0x00000000,0x002ff784,0x00000000,0x901d0000,0x299dffff,0x00000000,0x02000000,
    0x002ff7aa,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x76ff5e00,0x00000000,0x009cff5e,0x5e000000,
    0x000076ff,0x00000000,0x009cff39,0x00000000,0x00000000,0xaa020000,0x000051ff,0x00000000,
    0x84000000,0x000051ff,0xaa020000,0xf7aa029c,0x0000002f,0x0ee9cf04,0x00000000,0x00000000,
    0x07c2ff39,0x00000000,0x00000000,0x00000000,0xffaa0200,0x00000051,0x009ce71f,0x84000000,
    0x000051ff,0x0051ff84,0x39000000,0x000076ff,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x002fb626,0x00000000,0x00000000,0x1f000000,
    0x00004ec2,0x00000000,0x00000000,0x00000000,0xcf040000,0x00002ff7,0x0051ff5e,0x00000000,
    0x009ce71f,0xff390000,0x9cff439c,0x02000000,0x002ff7aa,0x5e000000,0x000076ff,0x0076ff5e,
    0x00000000,0x02000000,0x002ff7aa,0x39000000,0x0007c2ff,0x0ee9cf04,0x00000000,0x00000000,
    0x51ff8400,0x00000000,0x00000000,0x0076ff5e,0x00000000,0x02000000,0x002ff7aa,0x5e000000,
    0x000076ff,0x04000000,0x000ee9cf,0x00000000,0x00000000,0xff840000,0x02000051,0x022ff7aa,
    0x0054ffaa,0x00000000,0x2ff7aa02,0x00000000,0x02000000,0xe353f7aa,0x7aaa022f,0x022ff7aa,
    0xe753f7aa,0x5e000076,0x000076ff,0x0051ff84,0x1f000000,0x0007c2e7,0x0ee9cf04,0x02000000,
    0x002ff7aa,0x0051ff84,0x1f000000,0x0207c2e7,0x002ff7aa,0xaa020000,0x02002ff7,0x002ff7aa,
    0x00000000,0x00000000,0x04000000,0x000ee9cf,0x02000000,0x002ff7aa,0x84000000,0x000051ff,
    0x76ff8400,0x39000000,0x3907c2ff,0x1f009cff,0x002ff7e7,0x0051ff84,0xff390000,0xe9ff4de9,
    0x0000000e,0xf7e71f00,0xf7e71f2f,0x0000002f,0x00000000,0xe9ff3900,0x0000000e,0x5e000000,
    0x000076ff,0x00000000,0xf7aa0200,0x0000002f,0x00000000,0x00000000,0x07c2ff39,0x02000000,
    0x000ec9aa,0x04000000,0x0007c2ba,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0xc8681b00,0x99f7ffff,0x00000007,0x4351ff84,0x71f7ffe7,0x00000000,0x901d0000,
    0xc2ffffe7,0x00000029,0xe7430000,0x3971f7ff,0x0007c2ff,0xb91d0000,0x75f7ffff,0x00000007,
    0xffffe71f,0xffffffff,0x000076ff,0xe7430000,0x3971f7ff,0x0007c2ff,0x1f51ff84,0x51f7ffba,
    0x00000000,0xffff5e00,0x0051ffff,0x00000000,0xffff3900,0x07c2ffff,0x00000000,0x51ff8400,
    0x83000000,0x002ff7ff,0x5e000000,0x000076ff,0x00000000,0xba57ff84,0xba0476ff,0x00079dff,
    0x1f51ff84,0x51f7ffba,0x00000000,0xc8430000,0x54cdffff,0x00000000,0x4351ff84,0x71f7ffe7,
    0x00000000,0xe7430000,0x3971f7ff,0x0007c2ff,0xff5e0000,0xff900276,0x000799ff,0xb9420000,
    0xc2ffffff,0x0200002c,0xffffffaa,0xffffffff,0x000051ff,0x0051ff84,0xff390000,0x000007c2,
    0x009cff5e,0x02000000,0xaa54ffaa,0x00002ff7,0x00000000,0x0ee9e71f,0x9cffaa02,0x39000000,
    0x002ff7ff,0x009cff5e,0x00000000,0x0076ff84,0xffffaa02,0xffffffff,0x00002ff7,0x02000000,
    0x002ff7aa,0x00000000,0x02000000,0x0007c2aa,0x00000000,0x04000000,0x000ee9cf,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x04000000,0x000ee9cf,
    0x00000000,0x07c2aa02,0x51ff3900,0x02000000,0xffffffaa,0xffffffff,0x0ee9ffff,0x07c2e71f,
    0x0007bd1f,0x00000000,0xc9ffe743,0x4200002f,0x0029c9ba,0xe9cf0400,0x0000000e,0x00000000,
    0x39000000,0x000051ff,0x00000000,0x39000000,0x000076ff,0x00000000,0x00000000,0x0076ff39,
    0x00000000,0x901d0000,0x299dffff,0x00000000,0x02000000,0x002ff7aa,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x0ee9cf04,0x00000000,0x0076ff84,0x39000000,0x00009cff,0x00000000,0x009cff39,
    0x00000000,0x00000000,0xcf040000,0x000051ff,0x00000000,0xff690200,0x00000ee9,0xf7390000,
    0xf7aa022f,0x0000002f,0xffffcf04,0x4ec2ffff,0x00000000,0x0076ff5e,0x00000000,0x00000000,
    0x00000000,0xe9e71f00,0x0000000e,0x54ffaa02,0xe7420000,0x000007c2,0x0076ff84,0x39000000,
    0x00009cff,0x1f000000,0x0051ffe7,0x00000000,0x1f000000,0x0051ffe7,0x00000000,0x00000000,
    0xe7680200,0x002ff7ff,0x00000000,0x00000000,0x1f000000,0x75f7ffe7,0x00000007,0x00000000,
    0x00000000,0xffaa0200,0x0200009c,0x0007c2aa,0xf7ff9002,0x009ce799,0xff840000,0xe9e71f51,
    0x0200000e,0x002ff7aa,0xcf280000,0x000051ff,0x0051ff84,0x00000000,0x02000000,0x002ff7aa,
    0x04000000,0x000ee9cf,0x0ee9cf04,0x00000000,0x00000000,0x51ff8400,0x00000000,0x00000000,
    0x0051ff84,0x00000000,0x02000000,0x002ff7aa,0x5e000000,0x000076ff,0x04000000,0x000ee9cf,
    0x00000000,0x00000000,0xff840000,0x02000051,0xaa32f7aa,0x000076ff,0x00000000,0x2ff7aa02,
    0x00000000,0x02000000,0xaa32f7aa,0x32e31f76,0x022ff7aa,0xaa32f7aa,0x5e0007c2,0x020076ff,
    0x002ff7aa,0x04000000,0x000ee9cf,0x0ee9cf04,0x04000000,0x022ff7cf,0x002ff7aa,0x04000000,
    0x020ee9cf,0x002ff7aa,0xff690200,0x000007c2,0x079cff84,0x00000000,0x00000000,0x04000000,
    0x000ee9cf,0x02000000,0x002ff7aa,0x84000000,0x000051ff,0xc2ff3900,0x5e000007,0x1f0076ff,
    0x3907c2e7,0x0251ffff,0x0051ffaa,0xaa020000,0x51ffffff,0x00000000,0xff5e0000,0x76ff91a1,
    0x00000000,0x00000000,0x76ffcf04,0x00000000,0x5e000000,0x000076ff,0x00000000,0xff390000,
    0x0000009c,0x00000000,0x00000000,0x07c2ff39,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xffffaa02,0xffffffff,
    0x00000ec9,0xff99ff84,0xffffffff,0x0000009c,0xffe71f00,0xffffffff,0x000007c2,0xffff5e00,
    0xc6ffffff,0x0007c2ff,0xffe73900,0xffffffff,0x000007c2,0xffffe71f,0xffffffff,0x000076ff,
    0xffff5e00,0x99ffffff,0x0007c2ff,0xe778ff84,0xf7ffffff,0x0000002f,0xffff5e00,0x0051ffff,
    0x00000000,0xffff3900,0x07c2ffff,0x00000000,0x51ff8400,0xff830000,0x00002ff7,0x5e000000,
    0x000076ff,0x00000000,0xffffff84,0xff9cf7ff,0x0051ffff,0xe778ff84,0xf7ffffff,0x0000002f,
    0xffff5e00,0xffffffff,0x00000099,0xff99ff84,0xffffffff,0x0000009c,0xffff5e00,0xc6ffffff,
    0x0007c2ff,0xff5e0000,0xffffaa7a,0x0051ffff,0xffff5e00,0xffffffff,0x02000ee9,0xffffffaa,
    0xffffffff,0x000051ff,0x0051ff84,0xff390000,0x000007c2,0x0ee9e71f,0x1f000000,0x8407c2e7,
    0x000051ff,0x00000000,0x009cff39,0xffe71f00,0xcf040051,0x000076ff,0x2ff7cf04,0x1f000000,
    0x000ee9e7,0xffffaa02,0xffffffff,0x00002ff7,0x02000000,0x002ff7aa,0x00000000,0x02000000,
    0x0007c2aa,0x00000000,0x04000000,0x000ee9cf,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x04000000,0x000ee9cf,0x00000000,0x00000000,0x00000000,
    0x00000000,0xf75e0000,0xf75e002f,0x0000002f,0x0ee9cf04,0x0007bd1f,0x00000000,0x00000000,
    0xc9ba4200,0x0000002c,0xffba1d00,0x0000009c,0x00000000,0x00000000,0x00000000,0x00000000,
    0x5e000000,0x000051ff,0x00000000,0x00000000,0x07c2e71f,0x00000000,0x75e36802,0xe3687a69,
    0x00000775,0x02000000,0x002ff7aa,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x0076ff5e,0x00000000,
    0x0051ff84,0x39000000,0x00009cff,0x00000000,0x009cff39,0x00000000,0x00000000,0xff5e0000,
    0x00002ff7,0xe71f0000,0x9dffffff,0x00000029,0x9ccf0400,0xf7aa0200,0x0000002f,0xffffcf04,
    0xffffffff,0x00000076,0x4251ff84,0x99f7ffc8,0x00000007,0x00000000,0x9cff5e00,0x00000000,
    0xff830200,0x9dffffff,0x00000007,0x4ef7ff39,0xe7420000,0x00009cff,0x1f000000,0x0051ffe7,
    0x00000000,0x1f000000,0x0051ffe7,0x00000000,0x26000000,0xcdffffb9,0x1f000775,0xffffffe7,
    0xffffffff,0x002ff7ff,0xffc84302,0x002cc2ff,0x00000000,0x00000000,0xc2ffaa02,0x1f000007,
    0x00009ce7,0x022ff784,0x009cffaa,0xf7cf0400,0xffaa022f,0x02000051,0xffffffaa,0xf7ffffff,
    0x02000054,0x002ff7aa,0x00000000,0x02000000,0x002ff7aa,0x02000000,0x002ff7aa,0xffffcf04,
    0xffffffff,0x000051ff,0xffff8400,0xffffffff,0x02002ff7,0x002ff7aa,0x00000000,0x02000000,
    0xffffffaa,0xffffffff,0x000076ff,0x04000000,0x000ee9cf,0x00000000,0x00000000,0xff840000,
    0x02000051,0xffc8f7aa,0x00002ff7,0x00000000,0x2ff7aa02,0x00000000,0x02000000,0x842ff7aa,
    0x09c25e9c,0x022ff7aa,0x392ff7aa,0x5e002ff7,0x020076ff,0x002ff7aa,0x02000000,0x002ff7aa,
    0x0ee9cf04,0xaa1d0000,0x0207c2ff,0x002ff7aa,0x02000000,0x022ff7aa,0xffffffaa,0xc9ffffff,
    0x0000000e,0xffffe71f,0x002975c9,0x00000000,0x04000000,0x000ee9cf,0x02000000,0x002ff7aa,
    0x84000000,0x000051ff,0xe9cf0400,0xaa02000e,0x04002ff7,0x5e0ee9cf,0x0276c8f7,0x002ff7aa,
    0x1f000000,0x009cffe7,0x00000000,0xcf040000,0x0ee9ffff,0x00000000,0x00000000,0x07c2ff5e,
    0x00000000,0x5e000000,0x000076ff,0x00000000,0xaa020000,0x00002ff7,0x00000000,0x00000000,
    0x07c2ff39,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x22549002,0xaa1d0000,0x000076ff,0x7affff84,0xe7430007,
    0x00002ff7,0xc2ffcf04,0x26000029,0x00000796,0x76ffe71f,0xff430000,0x0007c2ff,0x54f7cf04,
    0xba1d0000,0x000051ff,0x1f000000,0x0007c2e7,0x00000000,0x7affcf04,0xe7430007,0x0007c2ff,
    0x54f7ff84,0xffaa1d00,0x00000076,0x00000000,0x0051ff84,0x00000000,0x00000000,0x07c2e71f,
    0x00000000,0x51ff8400,0xf7ff8300,0x0000002f,0x5e000000,0x000076ff,0x00000000,0x009cff84,
    0x07c2ff84,0x009cff5e,0x54f7ff84,0xffaa1d00,0x00000076,0x76ffe71f,0xe7430000,0x00002ff7,
    0x7affff84,0xe7430007,0x00002ff7,0x76ffcf04,0xff430000,0x0007c2ff,0xff5e0000,0x0029c9c6,
    0x004b4300,0x7affaa02,0x26000007,0x00000e8c,0xaa020000,0x00002ff7,0x00000000,0x0051ff84,
    0xff390000,0x000007c2,0x51ff8400,0x5e000000,0x5e0076ff,0x00009cff,0x00000000,0x0076ff5e,
    0xff390000,0xffaa32f7,0x0000009c,0x76ff8400,0x5e000000,0x00009cff,0x00000000,0xffe71f00,
    0x00000076,0x02000000,0x002ff7aa,0x00000000,0x02000000,0x0007c2aa,0x00000000,0x02000000,
    0x002ff7aa,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x04000000,0x000ee9cf,0x00000000,0x00000000,0x00000000,0x00000000,0xc2840000,0xc2840007,
    0x00000007,0xc9ff5e00,0x0007bd72,0x00000000,0x1d000000,0x004ec9ba,0x00000000,0xffffe71f,
    0x000076ff,0x00000000,0x00000000,0x00000000,0x00000000,0x84000000,0x000051ff,0x00000000,
    0x00000000,0x07c2cf04,0x00000000,0x0029c21f,0x1d00765e,0x00002fb9,0x02000000,0x002ff7aa,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x04000000,0x000ee9cf,0x00000000,0x1d51ff84,0x390ec9cf,0x00009cff,
    0x00000000,0x009cff39,0x00000000,0x00000000,0xffe71f00,0x00000076,0xe71f0000,0xc9ffffff,
    0x00000029,0x0ee95e00,0xf7aa0200,0x0000002f,0x00298c04,0xff831b00,0x00002ff7,0xff99ff84,
    0xffffffff,0x000007c2,0x00000000,0x2ff7cf04,0x00000000,0xffba1d00,0xc9ffffff,0x0000002f,
    0xffffaa02,0x99ffffff,0x00009cff,0x1f000000,0x0051ffe7,0x00000000,0x1f000000,0x0051ffe7,
    0x00000000,0xffe76802,0x00299dff,0x1f000000,0xffffffe7,0xffffffff,0x002ff7ff,0x1d000000,
    0xf7ffe790,0x00000775,0x00000000,0x0ee9ff84,0x39000000,0x1f0076ff,0x00009ce7,0x009cff39,
    0xc2ff3900,0xff5e0007,0x0200009c,0xffffffaa,0xf7ffffff,0x02000071,0x002ff7aa,0x00000000,
    0x02000000,0x002ff7aa,0x02000000,0x002ff7aa,0xffffcf04,0xffffffff,0x000051ff,0xffff8400,
    0xffffffff,0x02002ff7,0x002ff7aa,0xffcf0400,0x022ff7ff,0xffffffaa,0xffffffff,0x000076ff,
    0x04000000,0x000ee9cf,0x00000000,0x00000000,0xff840000,0x02000051,0xc8ffffaa,0x0007c2ff,
    0x00000000,0x2ff7aa02,0x00000000,0x02000000,0x392ff7aa,0x0276c8f7,0x022ff7aa,0x042ff7aa,
    0x5e0076cf,0x020076ff,0x002ff7aa,0x02000000,0x002ff7aa,0xffffcf04,0xffffffff,0x020051f7,
    0x002ff7aa,0x02000000,0x022ff7aa,0xffffffaa,0x29c9ffff,0x00000000,0xffba1d00,0xc9ffffff,
    0x00000029,0x04000000,0x000ee9cf,0x02000000,0x002ff7aa,0x84000000,0x000051ff,0xff840000,
    0xe71f0051,0x02000ee9,0x842ff7aa,0x0bc2849c,0x000ee9cf,0x1f000000,0x009cffe7,0x00000000,
    0x5e000000,0x0076ffff,0x00000000,0x1f000000,0x002ff7e7,0x00000000,0x5e000000,0x000076ff,
    0x00000000,0x39000000,0x00009cff,0x00000000,0x00000000,0x07c2ff39,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x39000000,0x00009cff,0x009cff84,0x5e000000,0x000076ff,0x07c2ff39,0x00000000,
    0x00000000,0x009cff39,0x84000000,0x0007c2ff,0x0076ff39,0x39000000,0x00009cff,0x1f000000,
    0x0007c2e7,0x00000000,0x009cff39,0x5e000000,0x0007c2ff,0x009cff84,0xff390000,0x0000009c,
    0x00000000,0x0051ff84,0x00000000,0x00000000,0x07c2e71f,0x00000000,0x51ff8400,0x2ff7ff83,
    0x00000000,0x5e000000,0x000076ff,0x00000000,0x0076ff84,0x009cff5e,0x009cff39,0x009cff84,
    0xff390000,0x0000009c,0x009cff5e,0x5e000000,0x000076ff,0x009cff84,0x5e000000,0x000076ff,
    0x009cff39,0x84000000,0x0007c2ff,0xff5e0000,0x00002ff7,0x00000000,0x54f7aa02,0x00000000,
    0x00000000,0xaa020000,0x00002ff7,0x00000000,0x0051ff84,0xff390000,0x000007c2,0x9cff3900,
    0xcf040000,0x1f002ff7,0x0207c2e7,0x020ee9aa,0x002ff7aa,0x84000000,0xe9ffffff,0x0000000e,
    0xe9e71f00,0xcf04000e,0x00002ff7,0x00000000,0xc2ffaa02,0x00000007,0x43000000,0x000ee9ff,
    0x00000000,0x02000000,0x0007c2aa,0x00000000,0x00000000,0x077aff84,0x02000000,0xffffc843,
    0x00074ec9,0x002f8102,0x00000000,0x00000000,0x00000000,0x02000000,0x000ee9aa,0x00000000,
    0x00000000,0x00000000,0x00000000,0x9ccf0400,0x76cf0400,0x00000000,0xe7430000,0x9dffffff,
    0x00000029,0xc9901d00,0x0000004e,0x02000000,0x392ff7aa,0x002ff7ff,0x009cff39,0x00000000,
    0x00000000,0x00000000,0xaa020000,0x00002ff7,0x00000000,0x00000000,0x0ee9cf04,0x00000000,
    0x00000000,0x0000765e,0x1f000000,0xffffffe7,0xffffffff,0x0076ffff,0x00000000,0x00000000,
    0x00000000,0xff5e0000,0x9cffffff,0x00000000,0x00000000,0x00000000,0x00000000,0x39000000,
    0x00009cff,0x00000000,0x1d51ff84,0x390ec9cf,0x00009cff,0x00000000,0x009cff39,0x00000000,
    0x00000000,0x9cffe71f,0x00000000,0x00000000,0xe75d0200,0x00002ff7,0x0076e71f,0xf7aa0200,
    0x0000002f,0x00000000,0x84000000,0x000076ff,0x54f7ff84,0xcf1d0000,0x000076ff,0x00000000,
    0x07c2ff39,0x00000000,0x4ef7cf04,0xe7420000,0x00002ff7,0xe7830200,0x3954f7ff,0x00009cff,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x1f000000,0x4ec9ffe7,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xba420000,0x002ff7ff,0x04000000,
    0x002ff7cf,0x5e000000,0x390051ff,0x000076ff,0x009ce71f,0x76ff5e00,0xe71f0000,0x020007c2,
    0x002ff7aa,0xba1d0000,0x020076ff,0x002ff7aa,0x00000000,0x02000000,0x002ff7aa,0x02000000,
    0x002ff7aa,0x0ee9cf04,0x00000000,0x00000000,0x51ff8400,0x00000000,0x02000000,0x002ff7aa,
    0xffcf0400,0x022ff7ff,0x002ff7aa,0x5e000000,0x000076ff,0x04000000,0x000ee9cf,0x00000000,
    0x00000000,0xff840000,0x02000051,0x049cffaa,0x0076ffcf,0x00000000,0x2ff7aa02,0x00000000,
    0x02000000,0x042ff7aa,0x0251ffcf,0x022ff7aa,0x002ff7aa,0x5e0ee984,0x020076ff,0x002ff7aa,
    0x02000000,0x002ff7aa,0xffffcf04,0xc2ffffff,0x02000029,0x002ff7aa,0x02000000,0x022ff7aa,
    0x002ff7aa,0xc2ff8402,0x00000007,0x00000000,0xffe77226,0x00000ee9,0x04000000,0x000ee9cf,
    0x02000000,0x002ff7aa,0x84000000,0x000051ff,0xff5e0000,0xff5e009c,0x0200009c,0xaa54ffaa,
    0x2fe93976,0x0007c2e7,0x84000000,0x2ff7ffff,0x00000000,0x04000000,0x000ee9cf,0x00000000,
    0xaa020000,0x00009cff,0x00000000,0x5e000000,0x000076ff,0x00000000,0x04000000,0x000ee9cf,
    0x00000000,0x00000000,0x07c2ff39,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xc8680200,0xffffffff,0x00009cff,
    0x0076ff84,0x39000000,0x00009cff,0x0076ff5e,0x00000000,0x00000000,0x0076ff5e,0x39000000,
    0x0007c2ff,0xffffff5e,0xffffffff,0x00009cff,0x1f000000,0x0007c2e7,0x00000000,0x0076ff5e,
    0x39000000,0x0007c2ff,0x0051ff84,0xff390000,0x0000009c,0x00000000,0x0051ff84,0x00000000,
    0x00000000,0x07c2e71f,0x00000000,0xcdff8400,0x0051f7ff,0x00000000,0x5e000000,0x000076ff,
    0x00000000,0x0051ff84,0x0076ff5e,0x009cff39,0x0051ff84,0xff390000,0x0000009c,0x0076ff5e,
    0x39000000,0x00009cff,0x0076ff84,0x39000000,0x00009cff,0x0076ff5e,0x39000000,0x0007c2ff,
    0xff5e0000,0x0000009c,0x00000000,0xffff5e00,0x4ec2f7ff,0x00000000,0xaa020000,0x00002ff7,
    0x00000000,0x0051ff84,0xff390000,0x000007c2,0xe9cf0400,0xff39000e,0x040007c2,0x1f2ff7cf,
    0x042ff7e7,0x000ee9cf,0x02000000,0x2ff7ffaa,0x00000000,0xffaa0200,0xff390051,0x000007c2,
    0x00000000,0x0ee9ff5e,0x00000000,0xffffaa02,0x000051f7,0x00000000,0x02000000,0x0007c2aa,
    0x00000000,0x00000000,0xffffba1d,0x1f002ff7,0xffffffe7,0xffffffff,0x002ff7ff,0x00000000,
    0x00000000,0x00000000,0x02000000,0x0007c2aa,0x00000000,0x00000000,0x00000000,0xcf040000,
    0xffffffff,0xffffffff,0x0007c2ff,0x00000000,0xe772c21f,0x02002ff7,0x0054cd83,0xffba1f00,
    0x390054f7,0x00009cff,0x07c2ff84,0x009cff39,0x00000000,0x00000000,0x00000000,0xaa020000,
    0x00002ff7,0x00000000,0x00000000,0x0ee9cf04,0x00000000,0x00000000,0x0000765e,0x1f000000,
    0xffffffe7,0xffffffff,0x0076ffff,0x00000000,0x00000000,0x00000000,0xff5e0000,0x9cffffff,
    0x00000000,0x00000000,0x00000000,0x00000000,0xaa020000,0x00002ff7,0x00000000,0x0076ff84,
    0x39000000,0x00009cff,0x00000000,0x009cff39,0x00000000,0x1d000000,0x079cffcf,0x00000000,
    0x00000000,0x5e000000,0x000076ff,0x000ee984,0xf7aa0200,0x0000002f,0x00000000,0x39000000,
    0x00009cff,0x0076ff84,0x39000000,0x00009cff,0x00000000,0x0076ff84,0x00000000,0x0076ff5e,
    0x39000000,0x00009cff,0x00000000,0x5e000000,0x00009cff,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x1f000000,0x4ec9ffe7,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0xb9420000,0x002ff7ff,0x1f000000,0x0007c2e7,0x5e000000,0x390051ff,
    0x000076ff,0x009ccf04,0x2ff7aa02,0xaa020000,0x02002ff7,0x002ff7aa,0x04000000,0x000ee9cf,
    0x0051ff84,0x00000000,0x02000000,0x002ff7aa,0x04000000,0x000ee9cf,0x0ee9cf04,0x00000000,
    0x00000000,0x51ff8400,0x00000000,0x00000000,0x0051ff84,0x02000000,0x022ff7aa,0x002ff7aa,
    0x5e000000,0x000076ff,0x04000000,0x000ee9cf,0x00000000,0x00000000,0xff840000,0x02000051,
    0x002ff7aa,0x2ff7ff39,0x00000000,0x2ff7aa02,0x00000000,0x02000000,0x002ff7aa,0x020ee984,
    0x022ff7aa,0x002ff7aa,0x5e51e71f,0x020076ff,0x002ff7aa,0x04000000,0x000ee9cf,0x0ee9cf04,
    0x00000000,0x02000000,0x002ff7aa,0x04000000,0x020ee9cf,0x002ff7aa,0xffaa0200,0x00000076,
    0x00000000,0xaa020000,0x000051ff,0x04000000,0x000ee9cf,0x02000000,0x002ff7aa,0x84000000,
    0x000051ff,0xe71f0000,0xff8407c2,0x00000051,0xe77dff84,0x72e31f51,0x00009cff,0xff390000,
    0xc2ff73e9,0x00000007,0x04000000,0x000ee9cf,0x00000000,0xff390000,0x00000ee9,0x00000000,
    0x5e000000,0x000076ff,0x00000000,0x00000000,0x0076ff5e,0x00000000,0x00000000,0x07c2ff39,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0xffffcf04,0xffffffff,0x0007c2ff,0x0051ff84,0x39000000,0x00009cff,
    0x0051ff84,0x00000000,0x00000000,0x0051ff84,0x39000000,0x0007c2ff,0xffffff84,0xffffffff,
    0x0007c2ff,0x1f000000,0x0007c2e7,0x00000000,0x0051ff84,0x39000000,0x0007c2ff,0x0051ff84,
    0xff390000,0x000007c2,0x00000000,0x0051ff84,0x00000000,0x00000000,0x07c2e71f,0x00000000,
    0xffff8400,0x07c2ffff,0x00000000,0x5e000000,0x000076ff,0x00000000,0x0051ff84,0x0076ff5e,
    0x07c2ff39,0x0051ff84,0xff390000,0x000007c2,0x0051ff84,0x39000000,0x00009cff,0x0051ff84,
    0x39000000,0x00009cff,0x0051ff84,0x39000000,0x0007c2ff,0xff5e0000,0x00000076,0x00000000,
    0xc8430000,0xffffffff,0x00000076,0xaa020000,0x00002ff7,0x00000000,0x0051ff84,0xff390000,
    0x000007c2,0xff840000,0xff840051,0x00000076,0x5e51ff84,0x3976aac2,0x0007c2ff,0x00000000,
    0x07c2ff5e,0x00000000,0xff390000,0xffaa09c2,0x00000051,0x39000000,0x0051ffff,0x00000000,
    0xffffaa02,0x000051f7,0x00000000,0x02000000,0x0007c2aa,0x00000000,0x00000000,0xffffba1d,
    0x1f002ff7,0x00002292,0xffff9026,0x000775f7,0x00000000,0x00000000,0x00000000,0x02000000,
    0x0007c2aa,0x00000000,0x00000000,0x00000000,0xcf040000,0xffffffff,0xffffffff,0x0007c2ff,
    0x00000000,0x3907bd1f,0x00009cff,0x0000001b,0x079ce71f,0x5e51ff5e,0x000076ff,0x9cffaa02,
    0x0051ff5e,0x00000000,0x00000000,0x00000000,0x84000000,0x000051ff,0x00000000,0x00000000,
    0x07c2cf04,0x00000000,0x00000000,0x00000000,0x00000000,0x02000000,0x002ff7aa,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0xff390000,0x0000009c,0x00000000,0x009cff5e,0x5e000000,0x000076ff,0x00000000,
    0x009cff39,0x00000000,0xcf1d0000,0x000799ff,0x00000000,0x00000000,0x39000000,0x04009cff,
    0xffffffcf,0xffffffff,0x000ee9ff,0x00000000,0x39000000,0x00009cff,0x0051ff5e,0x39000000,
    0x00009cff,0x04000000,0x000ee9cf,0x00000000,0x0051ff84,0x39000000,0x00009cff,0x00000000,
    0xaa020000,0x000076ff,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0xffe76802,0x00299dff,0x1f000000,0xffffffe7,0xffffffff,0x002ff7ff,0x1d000000,0xf7ffe790,
    0x00000775,0x1f000000,0x0007c2e7,0x5e000000,0x390051ff,0x000076ff,0x009ce71f,0xffffe71f,
    0xffffffff,0x020076ff,0x002ff7aa,0x02000000,0x002ff7aa,0x0076ff5e,0x00000000,0x02000000,
    0x002ff7aa,0x39000000,0x0007c2ff,0x0ee9cf04,0x00000000,0x00000000,0x51ff8400,0x00000000,
    0x00000000,0x0076ff5e,0x02000000,0x022ff7aa,0x002ff7aa,0x5e000000,0x000076ff,0x04000000,
    0x000ee9cf,0x00000000,0x00000000,0xff840000,0x02000051,0x002ff7aa,0xc2ff8400,0x00000007,
    0x2ff7aa02,0x00000000,0x02000000,0x002ff7aa,0x02000000,0x022ff7aa,0x002ff7aa,0x5e9caa02,
    0x000076ff,0x0051ff84,0x1f000000,0x0007c2e7,0x0ee9cf04,0x00000000,0x00000000,0x0051ff84,
    0x1f000000,0x0207c2e7,0x002ff7aa,0xff390000,0x000007c2,0x00000000,0x5e000000,0x000076ff,
    0x04000000,0x000ee9cf,0x02000000,0x002ff7aa,0x84000000,0x000051ff,0xaa020000,0xe9cf34f7,
    0x0000000e,0xe9c6ff5e,0xc6cf040e,0x000076ff,0xffaa0200,0xffaa0276,0x00000076,0x04000000,
    0x000ee9cf,0x00000000,0xffcf0400,0x00000051,0x00000000,0x5e000000,0x000076ff,0x00000000,
    0x00000000,0x0ee9cf04,0x00000000,0x00000000,0x07c2ff39,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x2cc9ff5e,
    0x39000000,0x0007c2ff,0x0076ff84,0x39000000,0x00009cff,0x0076ff5e,0x00000000,0x00000000,
    0x0076ff5e,0x39000000,0x0007c2ff,0x0051ff5e,0x00000000,0x00000000,0x1f000000,0x0007c2e7,
    0x00000000,0x0076ff5e,0x39000000,0x0007c2ff,0x0051ff84,0xff390000,0x000007c2,0x00000000,
    0x0051ff84,0x00000000,0x00000000,0x07c2e71f,0x00000000,0xf7ff8400,0x76ffcf51,0x00000000,
    0x5e000000,0x000076ff,0x00000000,0x0051ff84,0x0076ff5e,0x07c2ff39,0x0051ff84,0xff390000,
    0x000007c2,0x0076ff5e,0x39000000,0x00009cff,0x0076ff84,0x39000000,0x00009cff,0x0076ff5e,
    0x39000000,0x0007c2ff,0xff5e0000,0x00000076,0x00000000,0x00000000,0xff831b00,0x00000ee9,
    0xaa020000,0x00002ff7,0x00000000,0x0051ff84,0xff390000,0x000007c2,0xff390000,0xf7cf0bc2,
    0x0000002f,0x8476ff5e,0x5e9c5e76,0x000076ff,0x39000000,0x76ffffff,0x00000000,0xcf040000,
    0xe9e753f7,0x0000000e,0xcf040000,0x000076ff,0x00000000,0x43000000,0x000ee9ff,0x00000000,
    0x02000000,0x0007c2aa,0x00000000,0x00000000,0x079cff84,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x0ee98400,0x0ee98400,0x00000000,0x00000000,0x3907bd1f,0x00009cff,
    0x00000000,0x0051ff5e,0x399ce71f,0x0007c2ff,0xffe71f00,0x000099ff,0x00000000,0x00000000,
    0x00000000,0x5e000000,0x000051ff,0x00000000,0x00000000,0x07c2e71f,0x00000000,0x00000000,
    0x00000000,0x00000000,0x02000000,0x002ff7aa,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xf7aa0200,0x0000002f,
    0x00000000,0x0ee9e71f,0xaa020000,0x000051ff,0x00000000,0x009cff39,0x00000000,0xffcf0400,
    0x00000076,0x00000000,0x00000000,0x5e000000,0x04009cff,0xffffffcf,0xffffffff,0x000ee9ff,
    0x00000000,0x84000000,0x000076ff,0x0076e71f,0x39000000,0x00009cff,0x5e000000,0x00009cff,
    0x00000000,0x0076ff84,0x39000000,0x00009cff,0x00000000,0xff390000,0x00002ff7,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x26000000,0xcdffffb9,0x1f000775,
    0xffffffe7,0xffffffff,0x002ff7ff,0xffc84302,0x002cc2ff,0x00000000,0x1f000000,0x0007c2e7,
    0x39000000,0x1f0076ff,0x00009ce7,0x009cff39,0xffffff5e,0xffffffff,0x0207c2ff,0x002ff7aa,
    0x04000000,0x002ff7cf,0x0ee9e71f,0x00000000,0x02000000,0x002ff7aa,0x84000000,0x000076ff,
    0x0ee9cf04,0x00000000,0x00000000,0x51ff8400,0x00000000,0x00000000,0x0ee9e71f,0x02000000,
    0x022ff7aa,0x002ff7aa,0x5e000000,0x000076ff,0x04000000,0x000ee9cf,0x00000000,0x00000000,
    0xffaa0200,0x02000051,0x002ff7aa,0xffcf0400,0x00000076,0x2ff7aa02,0x00000000,0x02000000,
    0x002ff7aa,0x02000000,0x022ff7aa,0x002ff7aa,0x9bf75e00,0x000076ff,0x009cff39,0x5e000000,
    0x00009cff,0x0ee9cf04,0x00000000,0x00000000,0x009cff39,0x5e000000,0x02009cff,0x002ff7aa,
    0xaa020000,0x000051ff,0x00000000,0x84000000,0x000076ff,0x04000000,0x000ee9cf,0x00000000,
    0x002ff784,0xaa020000,0x00002ff7,0x84000000,0xc2ffc6ff,0x00000007,0xc2ffff39,0xffaa0207,
    0x000051ff,0xc2ff5e00,0xff390007,0x00000ee9,0x04000000,0x000ee9cf,0x00000000,0x9cff8400,
    0x00000000,0x00000000,0x5e000000,0x000076ff,0x00000000,0x00000000,0x76ff5e00,0x00000000,
    0x00000000,0x07c2ff39,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x0051ff84,0x5e000000,0x0007c2ff,0x009cff84,
    0x5e000000,0x000076ff,0x07c2ff39,0x00000000,0x00000000,0x009cff39,0x84000000,0x0007c2ff,
    0x009cff39,0x00000000,0x00000000,0x1f000000,0x0007c2e7,0x00000000,0x009cff39,0x84000000,
    0x0007c2ff,0x0051ff84,0xff390000,0x000007c2,0x00000000,0x0051ff84,0x00000000,0x00000000,
    0x07c2e71f,0x00000000,0x51ff8400,0xf7ff3900,0x0000002f,0x5e000000,0x000076ff,0x00000000,
    0x0051ff84,0x0076ff5e,0x07c2ff39,0x0051ff84,0xff390000,0x000007c2,0x009cff5e,0x5e000000,
    0x000076ff,0x009cff84,0x5e000000,0x000076ff,0x009cff39,0x84000000,0x0007c2ff,0xff5e0000,
    0x00000076,0x00000000,0x00000000,0xcf040000,0x00000ee9,0xaa020000,0x00002ff7,0x00000000,
    0x0076ff5e,0xff5e0000,0x000007c2,0xcf040000,0xc2ff72f7,0x00000007,0xcdc9ff39,0x9cdc1f2f,
    0x000051ff,0xcf040000,0xffe7a1ff,0x00000051,0x5e000000,0x9cffffff,0x00000000,0xff840000,
    0x000007c2,0x00000000,0x02000000,0x002ff7aa,0x00000000,0x02000000,0x0007c2aa,0x00000000,
    0x04000000,0x002ff7cf,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x009caa02,
    0x009caa02,0x00000000,0x002cb61f,0xaa09bd1f,0x00009cff,0x00000000,0x0051ff5e,0x049ce71f,
    0x29c9ffcf,0xff5e0000,0x00002ff7,0x00000000,0x00000000,0x00000000,0x39000000,0x000076ff,
    0x00000000,0x00000000,0x0076ff39,0x00000000,0x00000000,0x00000000,0x00000000,0x02000000,
    0x002ff7aa,0x00000000,0x1f000000,0x0051ffe7,0x00000000,0x00000000,0x00000000,0x00000000,
    0x1f000000,0x0051ffe7,0x00000000,0x9cff3900,0x00000000,0x00000000,0x9cffaa02,0xff690207,
    0x000007c2,0x00000000,0x009cff39,0x00000000,0x54f7cf04,0x00000000,0x00000000,0x00227583,
    0xe7430200,0x000051ff,0x00000000,0xf7aa0200,0x0000002f,0x00227583,0xff830200,0x00002ff7,
    0x54f7aa02,0xe7420000,0x000051ff,0xaa020000,0x000051ff,0x00000000,0x4ef7ff39,0xe7420000,
    0x000076ff,0x004e6800,0xffe74200,0x0000009c,0x1f000000,0x0051ffe7,0x00000000,0x1f000000,
    0x0051ffe7,0x00000000,0x00000000,0xe7680200,0x002ff7ff,0x00000000,0x00000000,0x1f000000,
    0x75f7ffe7,0x00000007,0x00000000,0x00000000,0x00000000,0x1f000000,0x00009ce7,0x022ff784,
    0x029cffaa,0x0051ffaa,0x04000000,0x020ee9cf,0x002ff7aa,0xaa1d0000,0x0007c2ff,0xc9ff8400,
    0x1b000029,0x02004b83,0x002ff7aa,0xffba2600,0x00000ee9,0x0ee9cf04,0x00000000,0x00000000,
    0x51ff8400,0x00000000,0x00000000,0xc9ff8400,0x1f000029,0x022ff7cf,0x002ff7aa,0x5e000000,
    0x000076ff,0x04000000,0x000ee9cf,0x02000000,0x00074e8e,0xf7ff4300,0x0200002f,0x002ff7aa,
    0xff390000,0x00002ff7,0x2ff7aa02,0x00000000,0x02000000,0x002ff7aa,0x02000000,0x022ff7aa,
    0x002ff7aa,0xffe71f00,0x000076ff,0x7affcf04,0xe7420007,0x000051ff,0x0ee9cf04,0x00000000,
    0x00000000,0x7affcf04,0xe7420007,0x02002ff7,0x002ff7aa,0x5e000000,0x00009cff,0x00247583,
    0xff690200,0x00002ff7,0x04000000,0x000ee9cf,0x00000000,0x29c9ff5e,0xff690200,0x00000ee9,
    0x39000000,0x76ffffff,0x00000000,0x9cffe71f,0xff5e0000,0x00002ff7,0x51ffcf04,0x84000000,
    0x00009cff,0x04000000,0x000ee9cf,0x00000000,0x2ff7ff39,0x00000000,0x00000000,0x5e000000,
    0x000076ff,0x00000000,0x00000000,0xe9cf0400,0x0000000e,0x00000000,0x07c2ff39,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x29c2ff5e,0xe75d0200,0x0007c2ff,0x7affff84,0xe7430007,0x00002ff7,0xc2ffcf04,
    0x26000029,0x00000796,0x76ffcf04,0xff430000,0x0007c2ff,0x99ffcf04,0x26000007,0x00005190,
    0x1f000000,0x0007c2e7,0x00000000,0x7affcf04,0xff430007,0x0007c2ff,0x0051ff84,0xff390000,
    0x000007c2,0x00000000,0x0051ff84,0x00000000,0x00000000,0x07c2e71f,0x00000000,0x51ff8400,
    0xff5e0000,0x000007c2,0x39000000,0x0029e9ff,0x00000000,0x0051ff84,0x0076ff5e,0x07c2ff39,
    0x0051ff84,0xff390000,0x000007c2,0x76ffe71f,0xff430000,0x00002ff7,0x7affff84,0xe7430007,
    0x00002ff7,0x76ffe71f,0xff430000,0x0007c2ff,0xff5e0000,0x00000076,0x00000000,0x074eb604,
    0xff690200,0x00000ee9,0x84000000,0x000799ff,0x00000000,0x29e9ff39,0xffe74200,0x000007c2,
    0x84000000,0x76ffffff,0x00000000,0xe9ffcf04,0xffaa020e,0x00000ee9,0xffaa0200,0xff5e009c,
    0x00000ee9,0x1f000000,0x2ff7ffe7,0x00000000,0xf7ff3900,0x0000002f,0x00000000,0x02000000,
    0x002ff7aa,0x00000000,0x02000000,0x0007c2aa,0x00000000,0x04000000,0x000ee9cf,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x04000000,0x000ee9cf,
    0x00000000,0x00000000,0x00000000,0x00000000,0x0076e71f,0x0051e71f,0x00000000,0xffffe71f,
    0xffffffff,0x00000ee9,0x00000000,0x079ce71f,0x0051ff5e,0xffffe739,0x9ce9ffff,0x0007c2ff,
    0x00000000,0x00000000,0x00000000,0x04000000,0x0007c2cf,0x00000000,0x00000000,0x002ff784,
    0x00000000,0x00000000,0x00000000,0x00000000,0x02000000,0x002ff7aa,0x00000000,0x1f000000,
    0x0051ffe7,0x00000000,0x00000000,0x00000000,0x00000000,0x1f000000,0x0051ffe7,0x00000000,
    0x2ff7aa02,0x00000000,0x00000000,0xffe71f00,0xffffffff,0x00000051,0xffff3900,0xffffffff,
    0x0007c2ff,0xffffff5e,0xffffffff,0x000051ff,0xffffff84,0xffffffff,0x0000009c,0x00000000,
    0xf7aa0200,0x0000002f,0xffffff84,0xffffffff,0x00000076,0xffff3900,0xffffffff,0x000007c2,
    0xe71f0000,0x00000ee9,0x00000000,0xffffaa02,0xffffffff,0x000007c2,0xffff8400,0xc9ffffff,
    0x0000000e,0x1f000000,0x0051ffe7,0x00000000,0x1f000000,0x0051ffe7,0x00000000,0x00000000,
    0x00000000,0x002fb626,0x00000000,0x00000000,0x1f000000,0x00004ec2,0x00000000,0x00000000,
    0x39000000,0x00009cff,0x02000000,0x000ee9aa,0xf7ff9002,0x049ce799,0x000ee9cf,0x00000000,
    0x0251ff84,0xffffffaa,0xffffffff,0x00002ff7,0xffaa0200,0xffffffff,0x020051ff,0xffffffaa,
    0xf7ffffff,0x0000002f,0xffffcf04,0xffffffff,0x0007c2ff,0x51ff8400,0x00000000,0x00000000,
    0xffaa0200,0xffffffff,0x0207c2ff,0x002ff7aa,0x5e000000,0x000076ff,0xffffcf04,0xffffffff,
    0x02000ee9,0xffffffaa,0x9cffffff,0x02000000,0x002ff7aa,0x84000000,0x0007c2ff,0xffffaa02,
    0xffffffff,0x022ff7ff,0x002ff7aa,0x02000000,0x022ff7aa,0x002ff7aa,0xff840000,0x000076ff,
    0xffff3900,0xffffffff,0x00000099,0x0ee9cf04,0x00000000,0x00000000,0xffff3900,0xffffffff,
    0x02000099,0x002ff7aa,0x04000000,0x002ff7cf,0xffffff84,0xffffffff,0x00000099,0x04000000,
    0x000ee9cf,0x00000000,0xffffaa02,0xffffffff,0x00000051,0x04000000,0x2ff7ffcf,0x00000000,
    0x51ffcf04,0xff390000,0x00002ff7,0x009cff84,0x04000000,0x0051ffcf,0x04000000,0x000ee9cf,
    0x02000000,0xffffffaa,0xffffffff,0x002ff7ff,0x5e000000,0x000076ff,0x00000000,0x00000000,
    0xff5e0000,0x00000076,0x00000000,0x07c2ff39,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xffffe71f,0x99ffffff,
    0x0007c2ff,0xff99ff84,0xffffffff,0x00000099,0xffe71f00,0xffffffff,0x000007c2,0xffff5e00,
    0xc6ffffff,0x0007c2ff,0xffe73900,0xffffffff,0x000051ff,0x1f000000,0x0007c2e7,0x00000000,
    0xffff5e00,0x99ffffff,0x0007c2ff,0x0051ff84,0xff390000,0x000007c2,0xffffff39,0xffffffff,
    0x0007c2ff,0x00000000,0x07c2e71f,0x00000000,0x51ff8400,0xaa020000,0x00009cff,0x04000000,
    0xffffffcf,0x000051ff,0x0051ff84,0x0076ff5e,0x07c2ff39,0x0051ff84,0xff390000,0x000007c2,
    0xffff5e00,0xffffffff,0x0000009c,0xff99ff84,0xffffffff,0x0000009c,0xffff5e00,0xc6ffffff,
    0x0007c2ff,0xff5e0000,0x00000076,0x00000000,0xffffcf04,0xffffffff,0x00000076,0x39000000,
    0xffffffff,0x000051ff,0xffffcf04,0xff99ffff,0x000007c2,0x39000000,0x2ff7ffff,0x00000000,
    0x9cffaa02,0xff840000,0x000007c2,0xe9ff5e00,0x8400000e,0x0007c2ff,0x00000000,0x07c2ff84,
    0x00000000,0xffffcf04,0xffffffff,0x00002ff7,0x02000000,0x002ff7aa,0x00000000,0x02000000,
    0x0007c2aa,0x00000000,0x04000000,0x000ee9cf,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x04000000,0x000ee9cf,0x00000000,0x00000000,0x00000000,
    0x00000000,0x002ff739,0x002ff739,0x00000000,0xc8902600,0x9df7ffff,0x00000029,0x00000000,
    0xffba1f00,0x000054f7,0xffb91d00,0x0229c9ff,0x0076ffaa,0x00000000,0x00000000,0x00000000,
    0x00000000,0x002ff784,0x00000000,0x04000000,0x0007c2cf,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x1f000000,0x002ff7e7,0x00000000,0x00000000,
    0x00000000,0x00000000,0x1f000000,0x0051ffe7,0x00000000,0x009cff39,0x00000000,0x00000000,
    0xba1d0000,0x2fc9ffff,0x00000000,0xffff3900,0xffffffff,0x0007c2ff,0xffffff5e,0xffffffff,
    0x000051ff,0xffb96802,0x4ec2ffff,0x00000000,0x00000000,0xf7aa0200,0x0000002f,0xffc8681b,
    0x29c2ffff,0x00000000,0xb91f0000,0x99f7ffff,0x00000007,0xff840000,0x00000076,0x00000000,
    0xc8680200,0x75f7ffff,0x00000007,0xff901d00,0x0799f7ff,0x00000000,0x1f000000,0x0051ffe7,
    0x00000000,0x1f000000,0x002ff7e7,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x39000000,0x00009cff,0x00000000,
    0x0076ff39,0x00000000,0x39000000,0x00009cff,0x00000000,0x029cff5e,0xffffffaa,0x9dcdffff,
    0x00000029,0x68020000,0xcdffffc8,0x02000754,0xffffffaa,0x0754c6ff,0x00000000,0xffffcf04,
    0xffffffff,0x0007c2ff,0x51ff8400,0x00000000,0x00000000,0x68020000,0xcdffffc8,0x02000054,
    0x002ff7aa,0x5e000000,0x000076ff,0xffffcf04,0xffffffff,0x00000ee9,0xffe7901d,0x0775cdff,
    0x02000000,0x002ff7aa,0x04000000,0x0076ffcf,0xffffaa02,0xffffffff,0x022ff7ff,0x002ff7aa,
    0x02000000,0x022ff7aa,0x002ff7aa,0xff390000,0x000076ff,0xba420000,0x54cdffff,0x00000000,
    0x0ee9cf04,0x00000000,0x00000000,0xba420000,0x9cffffff,0x02000000,0x002ff7aa,0x00000000,
    0x009cff5e,0xffc66802,0x4ec2ffff,0x00000000,0x04000000,0x000ee9cf,0x00000000,0xe7830200,
    0x29c2ffff,0x00000000,0x02000000,0x0ee9ffaa,0x00000000,0x2ff7aa02,0xe71f0000,0x1f000ee9,
    0x002ff7e7,0x00000000,0x07c2ff5e,0x04000000,0x000ee9cf,0x02000000,0xffffffaa,0xffffffff,
    0x002ff7ff,0x5e000000,0x000076ff,0x00000000,0x00000000,0xcf040000,0x00000ee9,0x00000000,
    0x07c2ff39,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0xffba1d00,0x392fc9ff,0x0007c2ff,0x4351ff84,0x71f7ffe7,
    0x00000000,0x901d0000,0xc2ffffe7,0x00000029,0xc8430000,0x3971f7ff,0x0007c2ff,0x901d0000,
    0xc2ffffff,0x0000004e,0x1f000000,0x0007c2e7,0x00000000,0xe7430000,0x3971f7ff,0x00009cff,
    0x0051ff84,0xff390000,0x000007c2,0xffffff39,0xffffffff,0x0007c2ff,0x00000000,0x07c2e71f,
    0x00000000,0x51ff8400,0x1f000000,0x0051ffe7,0x00000000,0xffffa91d,0x000051ff,0x0051ff84,
    0x0076ff5e,0x07c2ff39,0x0051ff84,0xff390000,0x000007c2,0xc8430000,0x54cdffff,0x00000000,
    0x4351ff84,0x71f7ffe7,0x00000000,0xe7430000,0x3971f7ff,0x0007c2ff,0xff5e0000,0x00000076,
    0x00000000,0xe7901d00,0x54cdffff,0x00000000,0x00000000,0xffffc85d,0x000051ff,0xffba1f00,
    0xff3954f7,0x000007c2,0x04000000,0x009cffcf,0x00000000,0x51ff5e00,0xff390000,0x0000009c,
    0x2ff7ff39,0x04000000,0x0076ffcf,0x00000000,0x0076ff84,0x00000000,0xffffcf04,0xffffffff,
    0x00002ff7,0x02000000,0x002ff7aa,0x00000000,0x02000000,0x0007c2aa,0x00000000,0x04000000,
    0x000ee9cf,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x0007bd1f,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x0076e71f,0x00000000,
    0x39000000,0x000076ff,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x5e000000,0x00009cff,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x02000000,0x002ff7aa,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x5e000000,0x00009cff,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x2ff78400,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0xffcf0400,0x00000051,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x5e000000,0x000076ff,
    0x00000000,0x00000000,0x5e000000,0x000076ff,0x00000000,0x07c2ff39,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x39000000,0x00009cff,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x07c2e71f,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x0051ff84,0x00000000,0x00000000,
    0x00000000,0x39000000,0x0007c2ff,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x04000000,0x002ff7cf,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x0076ff84,
    0x00000000,0x02000000,0x0007c2aa,0x00000000,0x43000000,0x000ee9e7,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x0007bd1f,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x0ee98400,0x00000000,0x84000000,0x000007c2,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x84000000,0x00002ff7,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x84000000,0x00002ff7,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0xf7830200,0x00000775,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xff390000,0x00002ff7,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x5e000000,0xc2ffffff,0x00000007,0x00000000,0x00000000,
    0x00000000,0xff840000,0x07c2ffff,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x22993700,
    0xe7420000,0x000051ff,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x02000000,0x009cff84,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x0051ff84,0x00000000,0x00000000,0x00000000,0x39000000,0x0007c2ff,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x84020000,0x00009cff,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0xffffff5e,0x00000ee9,0x02000000,0x0007c2aa,
    0x00000000,0xffffaa02,0x00009cff,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x0007bd1f,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x51e71f00,0x00000000,0xe71f0000,0x00000051,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0xcf040000,0x0000009c,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0xcf040000,0x0000009c,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x26000000,
    0xffffe7b9,0x000007c2,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x5e000000,0x00000076,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x5e000000,0xc2ffffff,0x00000007,0x00000000,0x00000000,0x00000000,0xff840000,0x07c2ffff,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0xffff3900,0xffffffff,0x0000009c,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xffffe71f,0x002ff7ff,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x0051ff84,
    0x00000000,0x00000000,0x00000000,0x39000000,0x0007c2ff,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0xffffaa02,0x000051ff,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0xffc85d00,0x00000ee9,0x02000000,0x0007c2aa,0x00000000,0xf7ffaa02,0x0000079d,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0xcf040000,0xffffffff,0xffffffff,0x00000ee9,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0xc8430200,0x75cdffff,0x00000007,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0xffffe71f,0x00004ecd,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x0051ff84,0x00000000,0x00000000,0x00000000,
    0x39000000,0x0007c2ff,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xf7ffaa02,
    0x00000054,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x02000000,0x0007c2aa,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0xcf040000,0xffffffff,0xffffffff,
    0x00000ee9,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x02000000,0x0007c2aa,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000
};
